"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const raw_body_1 = __importDefault(require("raw-body"));
const child_process_1 = require("child_process");
const stream_1 = require("stream");
class Process extends stream_1.Duplex {
    constructor(proc) {
        super();
        this.proc = proc;
        this.waiting = false;
        const promise = new Promise((resolve, reject) => proc.on("close", code => (code === 0 ? resolve() : reject(code))));
        this.then = promise.then.bind(promise);
        this.catch = promise.catch.bind(promise);
        this.finally = promise.finally.bind(promise);
        if (this.proc.stdin) {
            this.proc.stdin.once("finish", () => this.end());
            this.once("finish", () => this.proc.stdin.end());
        }
        if (this.proc.stdout) {
            this.proc.stdout.on("readable", () => {
                if (this.waiting) {
                    this.waiting = false;
                    this._read();
                }
            });
            this.proc.stdout.once("end", () => this.push(null));
        }
    }
    _read() {
        if (!this.proc.stdout)
            return;
        this.waiting = true;
        for (let buf = this.proc.stdout.read(); buf !== null; buf = this.proc.stdout.read()) {
            this.push(buf);
            this.waiting = false;
        }
    }
    _write(input, encoding, cb) {
        if (this.proc.stdin)
            this.proc.stdin.write(input, encoding, cb);
    }
    // tslint:disable-next-line
    end(...args) {
        if (this.proc.stdin)
            this.proc.stdin.end(...args);
        return this;
    }
    async toString(encoding = true) {
        const promise = raw_body_1.default(this, { encoding });
        await this;
        return promise;
    }
}
const sh = new Proxy({}, {
    get: (_, name) => (...args) => new Process(child_process_1.spawn(name, args, { stdio: ["pipe", "pipe", process.stderr] })),
});
module.exports = sh;
//# sourceMappingURL=index.js.map